@@ -25,8 +25,29 @@
 #define DLGS_H
 #include <gtk/gtk.h>
 /*----------------------------------------------------------------------------*/
+/**
+ * @brief  Select folder dialog.
+ *
+ * @param[in]  gw_parent  Dialog parent widget
+ * @return     Foler path or NULL.
+ */
 char *add_images_folder_dialog (GtkWindow *gw_parent);
 /*----------------------------------------------------------------------------*/
+/**
+ * @brief  Select images dialog.
+ *
+ * @param[in]  gw_parent  Dialog parent widget
+ * @return     List with images.
+ */
 GSList *add_images_dialog (GtkWindow *gw_parent);
 /*----------------------------------------------------------------------------*/
+/**
+ * @brief  Error message dialog.
+ *
+ * @param[in]  gw_parent  Dialog parent widget
+ * @param[in]  s_message  Message to show
+ * @return     none
+ */
+void message_dialog_error (GtkWindow *gw_parent, const char *s_message);
+/*----------------------------------------------------------------------------*/
 #endif
